<?php
/*
 * Plugin Name: Sello ChannelConnector
 * Plugin URI: https://sello.io
 * Description: Easily connect marketplace to your webshop using Sello
 * Author: Sello
 * Author URI: https://sello.io/
 * Version: 1.6.3
 * Text Domain: sello-channelconnector
 * Domain Path: /languages
 *
 * WC requires at least: 4.9.6
 * WC tested up to: 5.8
 *
 * Copyright (c) 2018-2020 Sell-o AB
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

if (!defined('ABSPATH')) {
    exit;
}

require_once dirname(__FILE__) . '/vendor/autoload.php';
require_once dirname(__FILE__) . '/src/version.php';

if (!class_exists('Sello_ChannelConnector')) {
    /**
     * Class Sello_ChannelConnector
     */
    class Sello_ChannelConnector
    {
        /**
         * The reference the *Singleton* instance of this class.
         *
         * @var $instance
         */
        protected static $instance;

        /**
         * Returns the *Singleton* instance of this class.
         *
         * @return self::$instance The *Singleton* instance.
         */
        public static function getInstance()
        {
            if (null === self::$instance) {
                self::$instance = new self();
            }

            return self::$instance;
        }

        /**
         * Protected constructor to prevent creating a new instance of the
         * *Singleton* via the `new` operator from outside of this class.
         */
        protected function __construct()
        {
            add_action('plugins_loaded', array($this, 'init'));

            $ig = new \Sello\ChannelConnector\Integration();
            $ig->refresh();
            $integrations = $ig->getAll();

            $items = [
                new \Sello\ChannelConnector\Settings\Product($integrations),
                new \Sello\ChannelConnector\Setup($integrations),
                new \Sello\ChannelConnector\Product($integrations),
                new \Sello\ChannelConnector\Account($integrations),
                new \Sello\ChannelConnector\Category($integrations),
                new \Sello\ChannelConnector\Order($integrations)
            ];

            foreach ($items as $item) {
                $item->init();
            }
        }

        /**
         * Init the plugin after plugins_loaded so environment variables are set.
         */
        public function init()
        {
            add_filter(
                'plugin_action_links_' . plugin_basename(__FILE__),
                array($this, 'pluginActionLinks')
            );
        }

        /**
         * Get setting link.
         *
         * @since 1.0.0
         *
         * @return string Setting link
         */
        public function getSettingLink()
        {
            $section_slug = 'kco';

            return admin_url(
                'admin.php?page=wc-settings&tab=checkout&section=' . $section_slug
            );
        }

        /**
         * Adds plugin action links
         *
         * @param array $links Plugin action link before filtering.
         *
         * @return array Filtered links.
         */
        public function pluginActionLinks($links)
        {
            $setting_link = $this->getSettingLink();
            $strSettings = __('Settings', 'sello-channelconnector');
            $strSupport = __('Support', 'sello-channelconnector');
            $plugin_links = array(
                '<a href="' . $setting_link . '">' . $strSettings . '</a>',
                '<a href="https://sello.io/">' . $strSupport . '</a>',
            );

            return array_merge($plugin_links, $links);
        }
    }

    add_action(
        "admin_init",
        function () {
            Sello_ChannelConnector::getInstance();
        }
    );

    add_action(
        "rest_api_init",
        function () {
            $ig = new \Sello\ChannelConnector\Integration();
            $ig->refresh();
            $integrations = $ig->getAll();

            $items = [
                new \Sello\ChannelConnector\Order($integrations),
                new \Sello\ChannelConnector\GetUpdatedProducts()
            ];

            foreach ($items as $item) {
                $item->initApi();
            }
        }
    );

    \register_activation_hook(
        __FILE__,
        function () {
            wp_remote_post(
                "https://wc-api.sello.io/install",
                [
                    "headers" => array("Content-type" => "application/json"),
                    "body" => json_encode(
                        [
                            "url" => home_url(),
                            "description" => get_option('blogdescription'),
                            "email" => get_option('admin_email'),
                            "lang" => get_locale(),
                            "name" => get_option('blogname')
                        ]
                    ),
                    "method" => "POST"
                ]
            );
        }
    );

    $admin = new \Sello\ChannelConnector\Settings\Admin();
    add_action('admin_menu', [$admin, 'initMenu']);
}
