<?php
namespace Sello\ChannelConnector;

if (!defined('ABSPATH') ) {
    exit;
}

class Order
{
    private $table;
    private $integrations;

    public function __construct($integrations)
    {
        global $wpdb;
        $this->table = $wpdb->prefix."sello_channelconnector_orders";
        $this->integrations = $integrations;
    }

    private function _check_exists()
    {
        global $wpdb;
        if ($wpdb->get_var("SHOW TABLES LIKE '{$this->table}'") != $this->table) {
            $wpdb->query(
                "CREATE TABLE `{$this->table}` (".
                "`sello_id` int(11) unsigned NOT NULL,".
                "`woo_id` int(11) unsigned NOT NULL,".
                "`created_at` timestamp NULL DEFAULT CURRENT_TIMESTAMP,".
                "PRIMARY KEY (`sello_id`)".
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8"
            );
        }
    }

    public function init()
    {
        global $wpdb;
        $this->_check_exists();

        add_action('woocommerce_admin_order_data_after_order_details', [$this, 'show_delivery_note_link'], 60, 1);
        add_filter('bulk_actions-edit-shop_order', [$this, 'add_order_bulk_actions'], 20, 1);
        add_filter('handle_bulk_actions-edit-shop_order', [$this, 'handle_order_bulk_actions'], 10, 3);
        add_action('admin_notices', [$this, 'downloads_bulk_action_admin_notice']);
    }

    function downloads_bulk_action_admin_notice()
    {
        if (empty($_REQUEST['urls'])) {
            return;
        }

        $urls = explode(",", $_REQUEST['urls']);
        echo '<div id="message" class="updated fade">';

        foreach ($urls as $n => $url) {
            if ($url) {
                echo '<p><a href="'.$url.'">'.__('Download packslip', 'channelconnector').' '.($n + 1).'</a></p>';
            }
        }

        echo '</div>';
    }

    function add_order_bulk_actions($actions)
    {
        $actions['cc-packslips'] = __('Download Packslips', 'channelconnector');
        return $actions;
    }

    // Make the action from selected orders
    function handle_order_bulk_actions( $redirect_to, $action, $post_ids)
    {
        if ($action !== 'cc-packslips') {
            return $redirect_to; // Exit
        }

        $processed_ids = array();
        $selloOrderIds = [];

        foreach ( $post_ids as $post_id ) {
            $selloOrderId = get_post_meta($post_id, 'sello_order_id', true);
            $selloOrderIds[] = $selloOrderId;
        }

        $ccAccountId = get_option('cc_account_id');
        $data = wp_remote_post(
            "https://wc-api.sello.io/orders/packslip",
            [
                "headers" => array("Content-type" => "application/json"),
                "body" => json_encode(
                    [
                        "accountId" => $ccAccountId,
                        "orderIds" => $selloOrderIds,
                    ]
                ),
                "method" => "POST"
            ]
        );
        $urls = json_decode(\wp_remote_retrieve_body($data), true);
        // Got invalid data back, or an WP_Error from wp_remote_get
        if (is_null($urls) || !is_array($urls)) {
            return $redirect_to;
        }
        $realUrls = [];
        foreach ($urls['urls'] as $u) {
            if ($u && $u != "null") {
                $realUrls[] = $u;
            }
        }

        if (count($realUrls) === 1) {
            return $redirect_to = $realUrls[0];
        } else {
            return add_query_arg(
                array(
                    'urls' => implode(',', $realUrls),
                ),
                $redirect_to
            );
        }
    }

    function show_delivery_note_link($order)
    {
        $selloOrderId = get_post_meta($order->get_id(), 'sello_order_id', true);
        echo '<br clear="all"><p><strong><a href="admin.php?page=channelconnector&action=getPackslip&orderId='.$selloOrderId.'" target="_blank">'.__('Download delivery note').'</a></strong></p>';
    }

    public function initApi()
    {
        register_rest_route(
            "wc-channelconnector/v1",
            "/orders", [
                "methods" => "POST",
                "callback" => [$this, "addOrder"],
                "permission_callback" => function ($request) {
                    return is_user_logged_in();
                }
            ]
        );
    }

    public function isRegistered($selloId)
    {
        global $wpdb;
        $this->_check_exists();
        $res = $wpdb->get_results(
            $wpdb->prepare("SELECT woo_id FROM ".$this->table." WHERE sello_id=%d", $selloId),
            ARRAY_A
        );

        return count($res) > 0 && $res[0]['woo_id'] ? $res[0]['woo_id'] : false;
    }

    public function setAsRegistered($selloId, $wooId)
    {
        global $wpdb;
        $this->_check_exists();
        $wpdb->query(
            $wpdb->prepare("INSERT INTO ".$this->table." (sello_id, woo_id) VALUES(%d, %d)", $selloId, $wooId)
        );
    }

    public function addOrder()
    {
        global $wpdb;
        $data = json_decode(file_get_contents("php://input"), true);

        $isRegistered = $this->isRegistered($data['id']);
        if ($isRegistered) {
            return new \WP_REST_Response(
                [
                    "id" => $isRegistered,
                    "exists" => true
                ],
                201
            );
        }

        if (is_null(\WC()->cart)) {
            \WC()->frontend_includes();
            \wc_load_cart();
        }

        // Get the integration
        foreach ($this->integrations as $ig) {
            if ($ig["id"] == $data["integration_id"]) {
                $integration = $ig;
            }
        }

        $address = array(
            'first_name' => $data["customer_first_name"],
            'last_name' => $data["customer_last_name"],
            'email' => $data["customer_email"],
            'phone' => $data["customer_phone"],
            'address_1' => $data["customer_address"],
            'address_2' => $data["customer_address_2"],
            'city' => $data["customer_city"],
            'state' => $data["customer_state"],
            'postcode' => $data["customer_zip"],
            'country' => $data["customer_country_code"]
        );
        // Status for new orders
        $newOrderStatus = \get_option('cc_new_order_status');
        // Now we create the order
        $order = \wc_create_order();
        if ($newOrderStatus) {
            $order->set_status($newOrderStatus);
        }
        $order->set_prices_include_tax(false);
        $order->set_currency($data['currency']);

        foreach ($data["rows"] as $row) {
            if ($row["woo_id"]) {
                $product = \wc_get_product($row["woo_id"]);
                if (!$product) {
                    continue;
                }

                $tax_rates = \WC_Tax::get_rates_from_location(
                    $product->get_tax_class('unfiltered'),
                    [
                        $data["customer_country_code"],
                        $data["customer_state"],
                        $data["customer_zip"],
                        $data["customer_city"]
                    ]
                );
                $taxes = \WC_Tax::calc_tax(
                    $row["price"] * $row["quantity"],
                    $tax_rates,
                    true
                );
                $price = \WC_Tax::round(
                    $row["price"] * $row["quantity"] - \wc_round_tax_total(array_sum($taxes))
                );

                $order->add_product(
                    $product,
                    $row["quantity"],
                    [
                        "subtotal" => $row["quantity"] > 0 ? $price/$row["quantity"] : 0,
                        "total" => $price
                    ]
                );
            }
        }

        $order->set_address($address, 'billing');
        $order->set_address($address, 'shipping');
        $order->add_meta_data("customer_alias", $data["customer_alias"]);
        $order->add_meta_data("sello_order_number", $data["number"]);
        $order->add_meta_data("sello_order_id", $data["id"]);
        $order->add_meta_data("integration_id", $data["integration_id"]);
        $order->add_meta_data("market_reference", $data["market_reference"]);

        // Shipping cost
        $shippingCost = $data["shipping_cost"];
        $shipping_rates = \WC_Tax::find_shipping_rates(
            [
                        'country' => $data["customer_country_code"],
                        'state' => $data["customer_state"],
                        'postcode' => $data["customer_zip"],
                        'city' => $data["customer_city"]
            ]
        );
        // Shipping taxes
        $shipping_taxes = \WC_Tax::calc_tax(
            $data["shipping_cost"],
            $shipping_rates,
            true
        );

        // Remove taxes from shipping cost
        foreach ($shipping_taxes as $t) {
            $shippingCost -= $t;
        }

        $currentShippingMethod = get_option('cc_shipping_method');
        // Default
        $shipping_name = "Shipping";
        // Marketplace name from Sello
        if (isset($data['market_name']) && !empty($data['market_name'])) {
            $shipping_name = $data['market_name'];
        }
        $shippingItem = new \WC_Order_Item_Shipping();
        $shippingItem->set_name($shipping_name);
        $shippingItem->set_total(\wc_format_decimal($shippingCost));
        $shippingItem->set_taxes($shipping_taxes);

        if ($currentShippingMethod) {
            $shippingItem->set_method_id($currentShippingMethod);
        }

        $order->add_item($shippingItem);

        if ($integration) {
            $order->add_meta_data("market", $integration["display_name"]);
            $orderDesc = "Order from ".$integration["display_name"];
        } else {
            $orderDesc = "Order from Sello";
        }

        // Should use default payment method?
        $useDefaultPaymentMethod = get_option('cc_default_payment_method', true);
        if ($useDefaultPaymentMethod === "false") {
            // Get payment method to use and set on order
            $currentPaymentMethod = get_option('cc_payment_method');
            $p = new \WC_Payment_Gateways();
            $paymentMethods = $p->payment_gateways();
            $paymentMethod = $paymentMethods[$currentPaymentMethod];
            $order->set_payment_method($paymentMethod);
        }

        if (!empty($data['delivery_note_urls'])) {
            $delivery_notes = $data['delivery_note_urls'];
            if (!is_array($delivery_notes)) {
                $delivery_notes = array($delivery_notes);
            }
            $order->add_meta_data('sello_delivery_notes', implode(",", $delivery_notes));
        }

        $order->calculate_totals();
        $order->save();
        if ($useDefaultPaymentMethod === "false") {
            $order->payment_complete();
        }

        $this->setAsRegistered($data['id'], $order->get_id());
        if (function_exists("call_min_pckasse")) {
            $order->reduce_order_stock();
            call_min_pckasse($order->get_id());
        }

        return new \WP_REST_Response(
            [
                "id" => $order->get_id()
            ],
            201
        );
    }

}
